<?php
/**
 * Controller
 * @package     Me
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Me_Controller
{
    /**
     * Controller名
     */
    protected $controller = 'Main';
    /**
     * Action名
     */
    protected $action;
    /**
     * Applicationオブジェクト
     * @var Me_Application
     */
    protected $app;
    /**
     * Logicオブジェクト
     * @var object
     */
    protected $logic;
    /**
     * Logicオブジェクト自動生成
     * @var boolean
     */
    protected $logic_load = true;
    /**
     * VIEWファイルパス
     * app/views 以下のパスを指定する
     * @var string
     */
    protected $view;
    /**
     * viewに渡す情報
     * @var array
     */
    protected $data = array();
    /**
     * リダイレクトパス
     * @var string
     */
    protected $redirect_path;
    /**
     * FLASHメッセージ
     * @var string
     */
    protected $flash;
    /**
     * トークン確認有無
     * @var boolean
     */
    protected $check_token = true;

    /**
     * Constructor
     * @param string $controller コントローラー名
     * @param string $action アクション名
     */
    public function __construct($action = 'index', $app = null)
    {
        $this->app = $app;

        $this->action = $action;
        // デフォルトのviewファイルをセット
        $this->view = '/' . $this->controller . '/' . $action . '.php';
        // logicオブジェクト作成
        if ($this->logic_load) {
            $logic_class = $this->controller . '_Logic';
            $this->logic = new $logic_class();
        }
    }

    /**
     * execute
     */
    public function execute()
    {
        // Actionメソッドの有無確認
        $action = $this->action;
        if (!method_exists($this, $action)) {
            throw new Me_Exception(4);
        }

        // Execute
        $this->prepend();
        $this->$action();
        $this->render();
        $this->append();
    }

    /**
     * actionの前処理
     */
    public function prepend()
    {
        // ユーザー認証処理
        $user_logic = new User_Logic();
        /* @var $login_user User */
        $login_user = $user_logic->authentication();
        $this->app->authenticated = true;

        // ライセンス認証
        License_Util::auth();

        // flash処理
        if (isset($_SESSION['_flash'])) {
            $this->set_data(array('_flash' => $_SESSION['_flash']));
            unset($_SESSION['_flash']);
        }

        // CSRF対策
        if ($this->check_token) {
            Me_Util::check_token();
        }

        $this->set_data(compact('login_user'));
    }

    /**
     * render
     * viewファイルをincludeして、$dataの値を展開して渡す
     */
    public function render()
    {
        // リダイレクト指定があれば、viewの表示は省略する。
        if (!empty($this->redirect_path)) {
            return;
        }

        // viewが指定されていなければ終了
        if (empty($this->view)) {
            return;
        }

        // $data内の変数を展開する
        extract($this->data);

        // VIEWファイルをインクルードする
        include('views' . $this->view);
    }

    /**
     * actionの後処理
     */
    public function append()
    {
        // DBコミット
        Me_Db::commit();

        // リダイレクト処理
        if (!empty($this->redirect_path)) {
            // flashメッセージをセット
            if (!empty($this->flash)) {
                $_SESSION['_flash'] = $this->flash;
            }
            // リダイレクト
            Me_Util::redirect($this->redirect_path);
        }
    }

    /**
     * $dataプロパティに配列を追加する
     * @param array $compact_data
     */
    protected function set_data($compact_data)
    {
        $this->data = array_merge($this->data, $compact_data);
    }
}