<?php
/**
 * Page Controller
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Page_Controller extends Me_Controller
{
    /**
     * Controller名
     */
    protected $controller = 'Page';
    /**
     * Logicオブジェクト
     * @var Page_Logic
     */
    protected $logic;

    /**
     * ページ編集
     */
    public function edit()
    {
        License_Util::check_valid();

        $page_id = Me_Util::get_query_string('pi');
        $template_id = Me_Util::get_query_string('ti');
        $path = Me_Util::get_query_string('path');
        $init = Me_Util::get_query_string('init');
        if (!strlen($page_id) && !strlen($template_id)) {
            throw new Me_Exception(403);
        }

        $_form_id = Me_Util::get_query_string('_form_id');
        if ($_form_id === null) {
            $_form_id = Me_Util::generate_form_id();
            $url = Me_Util::add_query_string($_SERVER["REQUEST_URI"], '_form_id=' . $_form_id);
            Me_Util::redirect($url);
        }

        // ページ情報取得
        $form = $this->logic->edit_page($page_id, $template_id);
        // 初期値セットフラグを反映
        if ($init === 'on') {
            $form->init = true;
        }
        // sessionに保存
        Me_Util::set_form($form, $_form_id);

        $is_edit = (!empty($form->_id)) ? true : false;
        $page_title = $this->logic->get_page_title($is_edit);
        if (empty($path)) {
            $path = $form->page_path;
        }

        $this->set_data(compact('form', 'is_edit', 'page_title', '_form_id', 'path'));
    }

    /**
     * ページ編集iframe
     */
    public function edit_iframe()
    {
        $form = Me_Util::get_form();
        $this->logic->output_edit_view($form->document, $form);
        $this->view = null;
    }

    /**
     * ヘッダー情報反映
     * プレビュー前に、編集内容を反映する
     */
    public function update_header()
    {
        /* @var $form Page_Form */
        $form = Me_Util::get_form();
        $form->title = Me_Util::get_post_data('title');
        $form->keywords = Me_Util::get_post_data('keywords');
        $form->description = Me_Util::get_post_data('description');
        // sessionに保存
        Me_util::set_form($form);
        $this->view = null;
    }

    /**
     * プレビュー表示
     */
    public function preview()
    {
        $form = Me_Util::get_form();
        $output = new Page_Output();
        $output->preview($form);
        $this->view = null;
    }

    /**
     * ファイル名確認
     */
    public function check_file_name()
    {
        $form = Me_Util::get_form();
        $form->file_name = Me_Util::get_post_data('file_name');
        $this->logic->check_file_name($form);
        $this->set_data(compact('form'));
    }
    
    /**
     * ページ登録処理
     */
    public function post()
    {
        License_Util::check_valid();

        $form = Me_Util::get_form();
        $mapping = array(
            'public_flag' => 'public_flag',
            'title' => 'title',
            'keywords' => 'keywords',
            'description' => 'description',
            'note' => 'note',
        );
        Me_Util::post_data_to_object($form, $mapping);

        $is_edit = (!empty($form->_id)) ? true : false;
        if (!$is_edit) {
            // 新規のばあい file_name も取得
            $form->file_name = Me_Util::get_post_data('file_name');
        }

        // ページ登録
        if ($this->logic->register_page($form)) {
            if ($is_edit) {
                $this->flash = 'ページ（ ' . $form->page_path . ' ）を更新しました。';
            } else {
                $this->flash = 'ページ（ ' . $form->page_path . ' ）を作成しました。';
            }
            $this->redirect_path = '../Main/index?reload=on&path=' . $form->page_path;

            // フォーム開放
            Me_Util::unset_form();
        } else {
            // 登録失敗
            $this->view = '/Page/edit.php';
            $page_title = $this->logic->get_page_title($is_edit);
            $_form_id = Me_Util::get_form_id();
            $this->set_data(compact('form', 'is_edit', 'page_title', '_form_id'));
        }
    }

    /**
     * ファイル同期確認
     */
    public function sync_file()
    {
        License_Util::check_valid();
        User_Role::require_designer();
        // 未登録のページリストを取得
        list($file_list, $delete_list) = $this->logic->get_sync_list();
        $this->set_data(compact('file_list', 'delete_list'));
    }

    /**
     * ファイル同期処理
     */
    public function post_sync_file()
    {
        License_Util::check_valid();
        User_Role::require_designer();
        // 未登録のページリストを取得
        list($file_list, $delete_list) = $this->logic->get_sync_list();

        // DBに登録
        $this->logic->insert_page_list($file_list);
        // DBから削除
        $this->logic->delete_page_list($delete_list);

        $this->set_data(compact('file_list', 'delete_list'));
    }

    /**
     * 各編集ボックスを表示する
     */
    public function open()
    {
        $id = Me_Util::get_post_data('id');
        $repeat_id = Me_Util::get_post_data('repeat_id');
        $repeat_no = Me_Util::get_post_data('repeat_no');
        $_form_id = Me_Util::get_form_id();
        /* @var $form Page_Form */
        $form = Me_Util::get_form($_form_id);
        $metag = $form->document->get_element_by_id($id);

        // 該当行取得
        $row = $form->get_row($id, $repeat_id, $repeat_no);

        // ソースとリッチテキストは実体参照の状態を保持するために、2重エスケープ状態にする。
        switch ($metag->node_type) {
            case 'source':
            case 'rich':
                $row['node_value'] = str_replace('&', '&amp;', $row['node_value']);
                break;
            default:
                break;
        }
        $this->set_data(compact('metag', 'row', 'repeat_no', '_form_id'));
        $this->view = '/Page/open_' . $metag->node_type . '.php';
    }

    /**
     * 編集ボックスの内容を反映する
     */
    public function reflect()
    {
        $id = Me_Util::get_post_data('id');
        $repeat_id = Me_Util::get_post_data('repeat_id');
        $repeat_no = Me_Util::get_post_data('repeat_no');
        $node_value = Me_Util::get_post_data('node_value');
        $url = Me_Util::get_post_data('url');
        $target = Me_Util::get_post_data('target');
        $anchor = Me_Util::get_post_data('anchor');
        /* @var $form Page_Form */
        $form = Me_Util::get_form();
        $metag = $form->document->get_element_by_id($id);

        // 各タイプ別のlogic実行
        $method = 'reflect_' . $metag->node_type;
        $metag_logic = new Metag_Logic();
        $metag_logic->$method($id, $node_value, $form, $metag, $repeat_no, $url, $target, $anchor);

        // 値を表示する。
        $row = $form->get_row($id, $repeat_id, $repeat_no);

        $this->set_data(compact('metag', 'row'));
        $this->view = '/Page/reflect_metag.php';
    }

    /**
     * repeat領域の操作
     * commandにより、前に追加、後に追加、削除を行う。
     */
    public function handling_repeat()
    {
        $repeat_id = Me_Util::get_post_data('repeat_id');
        $repeat_no = Me_Util::get_post_data('repeat_no');
        $command = Me_Util::get_post_data('command');
        /* @var $form Page_Form */
        $form = Me_Util::get_form();
        $metag_logic = new Metag_Logic();
        $metag_logic->handling_repeat($command, $repeat_id, $repeat_no, $form);

        $this->logic->output_edit_view($form->document, $form);
        $this->view = null;
    }

    /**
     * リンクページ選択
     */
    public function select_page()
    {
        $dir = Me_Util::get_post_data('dir', '/');
        $rows = $this->logic->select_page($dir);
        $this->set_data(compact('rows'));
    }
}