<?php
/**
 * User Controller
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class User_Controller extends Me_Controller
{
    /**
     * Controller名
     */
    protected $controller = 'User';
    /**
     * Logicオブジェクト
     * @var User_Logic
     */
    protected $logic;

    /**
     * ユーザー一覧
     */
    public function index()
    {
        User_Role::require_admin();
        $user_list = $this->logic->select_list();
        $this->set_data(compact('user_list'));
    }

    /**
     * ユーザー編集
     */
    public function edit()
    {
        License_Util::check_valid();
        $user_id = Me_Util::get_query_string('ui');

        if (!$this->logic->is_editable($user_id)) {
            throw new Me_Exception(608);
        }

        $is_edit = !empty($user_id);

        if ($is_edit) {
            // 編集
            $form = $this->logic->select_user($user_id, 'User_Form');
        } else {
            // 新規追加
            $form = new User_Form();
        }
        $page_title = $this->logic->get_page_title($is_edit);

        $this->set_data(compact('form', 'page_title', 'is_edit'));
    }

    /**
     * ユーザー登録
     */
    public function post()
    {
        License_Util::check_valid();
        $form = new User_Form();
        $mapping = array(
            '_id' => '_id',
            'user_name' => 'user_name',
            'password' => 'password',
            'confirm_password' => 'confirm_password',
            'user_role' => 'user_role',
            'display_name' => 'display_name',
            'note' => 'note',
        );
        Me_Util::post_data_to_object($form, $mapping);

        if (!$this->logic->is_editable($form->_id)) {
            throw new Me_Exception(608);
        }

        $is_edit = !empty($form->_id);

        if ($this->logic->register_user($form)) {
            // 登録成功
            if (User_Role::require_admin(false)) {
                $this->redirect_path = '../User/index';
            } else {
                $this->redirect_path = '../System/index';
            }
            if ($is_edit) {
                $this->flash = 'ユーザー（ ' . $form->user_name . ' ）を編集しました。';
            } else {
                $this->flash = 'ユーザー（ ' . $form->user_name . ' ）を追加しました。';
            }
        } else {
            // 登録失敗
            $this->view = '/User/edit.php';
            $page_title = $this->logic->get_page_title($is_edit);
            $this->set_data(compact('form', 'page_title', 'is_edit'));
        }
    }

    /**
     * ユーザー削除
     */
    public function delete()
    {
        License_Util::check_valid();
        User_Role::require_admin();
        $user_id = Me_Util::get_query_string('ui');
        $user = $this->logic->select_user($user_id);
        if (!$this->logic->is_deletable($user)) {
            throw new Me_Exception(601);
        }
        $this->set_data(compact('user'));
    }

    /**
     * ユーザー削除処理
     */
    public function post_delete()
    {
        License_Util::check_valid();
        User_Role::require_admin();
        $user_id = Me_Util::get_post_data('ui');
        $user = $this->logic->select_user($user_id);
        if (!$this->logic->is_deletable($user)) {
            throw new Me_Exception(601);
        }
        $this->logic->delete_user($user);

        $this->redirect_path = '../User/index';
        $this->flash = 'ユーザー（ ' . $user->user_name . ' ）を削除しました。';
    }
}