<?php
/**
 * Metag Textarea
 * @package     Metag
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Metag_Textarea extends Metag
{
    /** editor cols */
    const EDITOR_COLS = '40';
    /** editor rows */
    const EDITOR_ROWS = '3';

    /**
     * タグタイプ
     * @var string
     */
    public $node_type = 'textarea';
    /**
     * 表示名
     * @var string
     */
    public $display_name = 'テキストエリア (textarea)';
    /**
     * ブロック要素かどうか
     * @var boolean
     */
    public $is_block = false;
    /**
     * cols attribute
     * @var string
     */
    public $cols;
    /**
     * rows attribute
     * @var string
     */
    public $rows;
    /**
     * style attribute
     * @var string
     */
    public $style;
    /**
     * 属性リスト
     * @var array
     */
    public $attr_list = array(
        'id' => 'id',
        'initial_value' => 'initial_value',
        'cols' => 'cols',
        'rows' => 'rows',
        'style' => 'style',
        'anchor' => 'anchor',
        'ref_id' => 'ref_id',
        'ref_page_id' => 'ref_page_id',
        'ref_link' => 'ref_link',
    );

    /**
     * 編集viewの開始タグ出力
     * @param array $row
     */
    public function edit_view($row)
    {
        // 参照指定がある場合は子HTMLを出力
        if (strlen($this->ref_id) > 0) {
            $child = $this->get_child_node();
            echo $child->node_value;
            return;
        }

        h::no_escape('<span class="metag" onclick="' . $this->get_open_method($row) . '"'
                . $this->get_style($row['repeat_no']) . '>'
                . $this->output($row, true)
                . '</span>');
    }

    /**
     * 出力
     * @param array $row
     * @param boolean $edit_mode
     * @return string
     */
    public function output($row, $edit_mode = false)
    {
        $ret = '';
        if ($edit_mode && (strlen($row['node_value']) < 1 || preg_match('/^[[:cntrl:]]+$/', $row['node_value']))) {
            $ret = Metag_Logic::BLANK;
        } else {
            $ret = Me_Util::nl2br_escape($row['node_value']);
        }
        $this->set_url($row);
        list($a_open, $a_close) = $this->get_a($row, $a_attr);
        $anchor = $this->get_anchor($row);
        return $anchor . $a_open . $ret . $a_close;
    }

    /**
     * editor
     */
    public function print_editor_attr()
    {
        $str = '';
        $cols = (empty($this->cols)) ? self::EDITOR_COLS : $this->cols;
        $str .= ' cols="' . Me_Util::html_escape($cols) . '"';
        $rows = (empty($this->rows)) ? self::EDITOR_ROWS : $this->rows;
        $str .= ' rows="' . Me_Util::html_escape($rows) . '"';
        if (!empty($this->style)) {
            $str .= ' style="' . Me_Util::html_escape($this->style) . '"';
        }
        h::no_escape($str);
    }

    /**
     * タグ編集
     */
    public function set_post_data()
    {
        $mapping = $this->attr_list;
        $mapping['selected_text'] = 'selected_text';
        Me_Util::post_data_to_object($this, $mapping);
    }
}